# Generated by Django 4.1.1 on 2022-09-13 09:26

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import drf_fileupload.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FileModel',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', models.FileField(upload_to=drf_fileupload.models.get_upload_to, verbose_name='the file object')),
                ('size', models.IntegerField(editable=False, verbose_name='the filesize')),
                ('name', models.CharField(editable=False, max_length=50, verbose_name='the filename')),
                ('md5', models.CharField(db_index=True, editable=False, max_length=32, unique=True, verbose_name='the md5 of the file')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='the created time of the file')),
                ('mimetype', models.CharField(editable=False, max_length=20, verbose_name='the mime type of the file')),
                ('created_by', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL, verbose_name='the owner of the file')),
            ],
            options={
                'ordering': ('-id',),
            },
        ),
    ]



from django.urls import path, include
from rest_framework import routers

from .views import FileViewSet, FileCleanView


router = routers.DefaultRouter()
router.register('', FileViewSet, basename='file-upload')

urlpatterns = [
    path('clean', FileCleanView.as_view(), name='file-clean')
] + router.urls



import humanfriendly

from django.conf import settings
from rest_framework import serializers
from .models import FileModel


class FileSerializer(serializers.ModelSerializer):

    created_by = serializers.HiddenField(default=serializers.CurrentUserDefault())

    class Meta:
        model = FileModel
        fields = '__all__'

    def validate_file(self, data):

        max_size = settings.MAX_FILE_UPLOAD if hasattr(settings, 'MAX_FILE_UPLOAD') else None
        if max_size and data.size > humanfriendly.parse_size(str(max_size)):
            raise serializers.ValidationError(f'file size limit {max_size}')

        return data



import os
import hashlib
from pathlib import Path
from datetime import datetime
from functools import partial

import magic
from django.db import models
from django.conf import settings


def md5sum(file, chunksize=4096):
    """calculate the md5 of file
    """
    md5 = hashlib.md5()
    for chunk in iter(partial(file.read, chunksize), b''):
        md5.update(chunk)
    return md5.hexdigest()


def get_upload_to(instance, filename):
    """custom upload_to
    """
    upload_to = settings.FILE_UPLOAD_TO if hasattr(settings, 'FILE_UPLOAD_TO') else settings.MEDIA_ROOT
    name = instance.name.rsplit('.', 1)
    ext = f'.{name[1]}' if len(name) == 2 else ''
    filename = Path(upload_to).joinpath(f'{instance.md5}{ext}')
    return datetime.now().strftime(str(filename))


class FileModel(models.Model):
    file = models.FileField(
        verbose_name='the file object', upload_to=get_upload_to)
    size = models.IntegerField(
        verbose_name='the filesize',
        editable=False)
    name = models.CharField(
        verbose_name='the filename',
        max_length=50,
        editable=False)
    md5 = models.CharField(
        verbose_name='the md5 of the file',
        unique=True,
        db_index=True,
        max_length=32,
        editable=False)
    created_at = models.DateTimeField(
        verbose_name='the created time of the file',
        auto_now_add=True)
    created_by = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        verbose_name='the owner of the file',
        null=True,
        on_delete=models.SET_NULL)
    mimetype = models.CharField(
        verbose_name='the mime type of the file',
        max_length=20,
        editable=False)

    def save(self, *args, **kwargs):
        """add informations before save
        """
        md5 = md5sum(self.file)
        self.md5 = md5
        self.size = self.file.size
        self.name = self.file.name
        obj = FileModel.objects.filter(md5=md5).first()
        if not obj:
            self.file.seek(0)
            self.mimetype = magic.from_buffer(self.file.read(4096), mime=True)
            super().save(*args, **kwargs)
        else:
            self.id = obj.id
            self.file = obj.file
            self.mimetype = obj.mimetype
            self.created_at = obj.created_at

    class Meta:
        ordering = ('-id',)



from ast import Mult, parse
from rest_framework import viewsets, permissions, views, parsers
from rest_framework.response import Response

from .models import FileModel
from .serializers import FileSerializer


class FileViewSet(viewsets.ModelViewSet):
    queryset = FileModel.objects.all()
    serializer_class = FileSerializer
    # parser_classes = [parsers.MultiPartParser, parsers.FormParser]

    def get_permissions(self):
        if self.action in ('list', 'retrieve'):
            return [permissions.AllowAny()]
        elif self.action in ('destroy',):
            return [permissions.IsAdminUser()]
        else:
            return [permissions.IsAuthenticated()]

    def create(self, request, *args, **kwargs):
        print(request.FILES)

        
        return super().create(request, *args, **kwargs)

    def destroy(self, request, *args, **kwargs):
        obj = self.get_object()
        obj.file.delete(save=False)
        obj.delete()
        return Response({'msg': f'the file `{obj.name}` has been deleted.'})


class FileCleanView(views.APIView):
    permission_classes = [permissions.IsAdminUser]

    def get(self, request):
        return Response({'msg': 'all uploaded files will be deleted, be carefully!'})

    def post(self, request):
        """delete all files"""
        for obj in FileModel.objects.all():
            obj.file.delete(save=False)
            obj.delete()
        return Response({'msg': 'completed'})
